#include <cstdio>
#include "miosix.h"

using namespace std;
using namespace miosix;

typedef Gpio<GPIOA_BASE,7> mosi;
typedef Gpio<GPIOA_BASE,6> miso;
typedef Gpio<GPIOA_BASE,5> sck;
typedef Gpio<GPIOE_BASE,3> cs;

void spiSend(unsigned char byte)
{
    SPI1->DR=byte;
    while((SPI1->SR & SPI_SR_RXNE)==0) ; //Wait
    byte=SPI1->DR; //Dummy read, causa l'azzeramento del bit RXNE
}

unsigned char spiReceive()
{
    SPI1->DR=0; //Dummy write, causa la ricezione di un byte dalla SPI
    while((SPI1->SR & SPI_SR_RXNE)==0) ; //Wait
    return SPI1->DR;
}

void accelInit()
{
    cs::low();
    spiSend(0x20);
    spiSend(0x47);
    cs::high();
}

// axis==0 : read X axis
// axis==1 : read Y axis
// axis==2 : read Z axis
signed char accelRead(int axis)
{
    cs::low();
    spiSend(0x80 | (0x29+2*axis));
    signed char result=spiReceive();
    cs::high();
    return result;
}

int main()
{
    mosi::mode(Mode::ALTERNATE);
    mosi::alternateFunction(5);
    miso::mode(Mode::ALTERNATE);
    miso::alternateFunction(5);
    sck::mode(Mode::ALTERNATE);
    sck::alternateFunction(5);
    cs::mode(Mode::OUTPUT);
    cs::high();
    RCC->APB2ENR |= RCC_APB2ENR_SPI1EN;
    SPI1->CR1 = SPI_CR1_SSM
              | SPI_CR1_SSI
              | SPI_CR1_MSTR
              | SPI_CR1_BR_2
              | SPI_CR1_SPE;
    accelInit();
    for(;;)
    {
        usleep(50000);
        int x=accelRead(0);
        int y=accelRead(1);
        int z=accelRead(2);
        printf("%d,%d,%d\n",x,y,z);
    }
}
