#include "stm32accel.h"
#include "ui_stm32accel.h"
#include "poulabout.h"
#include "stm32plot.h"

#include <QMessageBox>
#include <QStatusBar>
#include <QStringList>

#include <qwt_plot.h>
#include <qwt_plot_canvas.h>
#include <qwt_plot_curve.h>

STM32Accel::STM32Accel(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::STM32Accel)
{
    ui->setupUi(this);

    trayIcon = new QSystemTrayIcon(QIcon(":/poul_icon.png"), this);
    trayIcon->show();

    trayMenu = new QMenu(this);
    trayMenu->addAction(ui->actionConnect);
    trayMenu->addAction(ui->actionDisconnect);
    trayMenu->addSeparator();
    trayMenu->addAction(ui->actionAbout);
    trayMenu->addAction(ui->actionExit);

    trayIcon->setContextMenu(trayMenu);

    portName = "/dev/ttyUSB0";
    serialPort = new QextSerialPort(QextSerialPort::EventDriven, this);
    serialPort->setQueryMode(QextSerialPort::EventDriven);
    serialPort->setPortName(portName);
    serialPort->setBaudRate(BAUD19200);
    serialPort->setFlowControl(FLOW_OFF);
    serialPort->setParity(PAR_NONE);
    serialPort->setDataBits(DATA_8);
    serialPort->setStopBits(STOP_2);

    connect(serialPort, SIGNAL(readyRead()), this, SLOT(onReadyRead()));

    ui->centralWidget->setEnabled(false);

    ui->menuBar->hide();
}

STM32Accel::~STM32Accel()
{
    delete ui;
}

void STM32Accel::onReadyRead()
{
    const char separator = ',';
    const double scalingFactor = 127/2.3;

    if(serialPort->canReadLine()) {

        QString str = serialPort->readLine().trimmed();
        QStringList values = str.split(separator);

        if(values.count() == 3) {

            int x, y, z;
            bool okX, okY, okZ;

            x = values[0].toInt(&okX);
            y = values[1].toInt(&okY);
            z = values[2].toInt(&okZ);


            if(okX && okY && okZ) {
                ui->plot->addValue(x/scalingFactor,y/scalingFactor,z/scalingFactor);
            }
        }
    }
}

void STM32Accel::on_actionAbout_triggered()
{
    POuLAbout dialog(this);
    dialog.exec();
}

void STM32Accel::on_actionExit_triggered()
{
    ui->plot->stop();
    this->close();
}

void STM32Accel::on_actionConnect_triggered()
{
    if(!serialPort->isOpen()) {

        if (serialPort->open(QIODevice::ReadWrite)) {
            ui->statusBar->showMessage(tr("Board collegata"), 5000);

            ui->centralWidget->setEnabled(true);
            ui->actionConnect->setEnabled(false);
            ui->actionDisconnect->setEnabled(true);
            ui->plot->start();
        } else {
            QMessageBox::critical(this, tr("Errore"), tr("Impossibile aprire la porta: ") + portName);
        }
    }
}

void STM32Accel::on_actionDisconnect_triggered()
{
    if(serialPort->isOpen()) {

        serialPort->close();
        ui->statusBar->showMessage(tr("Board scollegata"), 5000);

        ui->centralWidget->setEnabled(false);
        ui->actionConnect->setEnabled(true);
        ui->actionDisconnect->setEnabled(false);
        ui->plot->stop();
    }
}
